package com.mortals.xhx.common.utils;

import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.exception.AppException;
import lombok.extern.slf4j.Slf4j;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.Buffer;
import java.nio.ByteBuffer;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.util.HashMap;
import java.util.Locale;

/**
 * 微信扫码二维码生成器
 */
@Slf4j
public class WXCreateQRCodeUtil {

    private static String ACCESS_TOKEN = "access_token";
    private static String ACCESS_TOKEN_URL = "http://172.15.28.113:8914/cgi-bin/token?grant_type=client_credential&appid=%s&secret=%s";
    private static String CREATE_QRCODE_URL = "http://172.15.28.113:8914/cgi-bin/wxaapp/createwxaqrcode?access_token=%s";

    /***
     *
     * @param path
     * @param appID
     * @param appSecret
     * @param qRCodeFilePath
     */
    public static String createWeiXinQRCode(String path,String appID,String appSecret,String qRCodeFilePath){

        String accessTokenUrl = String.format(ACCESS_TOKEN_URL,appID,appSecret);
        try {
            String rsp = HttpUtil.get(accessTokenUrl,new HashMap<>());
//            System.out.println("获取token:");
//            System.out.println(rsp);
            JSONObject jsonObject = JSONObject.parseObject(rsp);
            if(jsonObject.containsKey(ACCESS_TOKEN)){
                String accessToken = jsonObject.getString(ACCESS_TOKEN);
                String createwxaqrcode = String.format(CREATE_QRCODE_URL,accessToken);
                HashMap<String, Object> paramsMap = new HashMap<>();
                paramsMap.put("path",path);
                paramsMap.put("width",430);
                HttpResponse response = HttpRequest.post(createwxaqrcode).body(JSONObject.toJSONString(paramsMap)).execute();
                // 获取响应状态码
                int status = response.getStatus();
                if(status==200) {
                    // 获取响应的字节流
                    InputStream inputStream = response.bodyStream();
                    saveImageFromInputStream(inputStream, qRCodeFilePath);
                    return qRCodeFilePath;
                }else {
                    log.info("请求微信二维码创建失败，失败响应:{}",response.body());
                    return null;
                }
            }else {
                log.info("获取微信ACCESS_TOKEN失败");
                return null;
            }
        }catch (Exception e){
            log.info("创建二维码发生异常:{}",e.getMessage());
            return null;
        }
    }

    public static void saveImageFromInputStream(InputStream inputStream, String outputPath) throws IOException {
        try (FileOutputStream outputStream = new FileOutputStream(outputPath)) {
            byte[] buffer = new byte[1024];
            int bytesRead;
            while ((bytesRead = inputStream.read(buffer)) != -1) {
                outputStream.write(buffer, 0, bytesRead);
            }
        }
    }

//    public static void main(String[] args) {
//        String ss = createQRCode("/pages/index/index?query=123","wxd3d6df0ebaf88f98","fe4c83e34f89956960aa0f1f9e8f38b1");
//
//        System.out.println(ss);
//    }
}
